import os
import sys
import getopt
import cPickle as pickle
from datetime import datetime

from webxcreta import harvesters
from webxcreta.base import XcretaRun
from webxcreta.utils import log

filename = 'var/harvested_mbox_data'
pkl = filename + '.pkl'
mboxfile = "/Users/oubiwann/Library/Mail/Mailboxes/2006/SchroBox.mbox/mbox"
no_entries = False
title_count = 30
entry_count = 6

for option, arg in getopt.getopt(sys.argv[1:], "fm:h:t:e:", 
    ["flush", "mbox=", "harvest=", "titles=", "entries=", "no-entries" ])[0]:
    if option in ('-f', '--flush'):
        date = datetime.now().strftime("%Y%m%d")
        time = datetime.now().strftime("%H%M%S")
        backup = "%s_%s.%s.pkl" % (filename, date, time)
        try:
            os.rename(pkl, backup)
            log.info("Renamed '%s' to '%s'." % (pkl, backup))
        except OSError:
            log.info("Nothing to flush.")
    if option == '--no-entries':
        no_entries = True
    if option in ('-m', '--mbox'):
        mboxfile = arg
    if option in ('-h', '--harvest'):
        pkl = arg
    if option in ('-t', '--titles'):
        title_count = int(arg)
    if option in ('-e', '--entries'):
        entry_count = int(arg)

x = XcretaRun()
if not os.path.exists(pkl):
    log.info('Getting fresh mbox data...')
    x.harvest = harvesters.MboxHarvester(mboxfile)
    # save the data for potential future use
    pickle.dump(x.getPicklableHarvest(), open(pkl, 'w+'))
else:
    log.info('Getting havested info from old data...')
    x.harvest = pickle.load(open(pkl))
# Now print out some results
x.printTitles(title_count)
if not no_entries:
    x.printEntries(entry_count)
