import os
import sys
import getopt
import cPickle as pickle
from random import choice
from datetime import datetime

from webxcreta import clients
from webxcreta import harvesters
from webxcreta.base import XcretaRun
from webxcreta.utils import log

# config
filename = 'var/harvested_data'
pkl = filename + '.pkl'
defaultCount = 500
doEntries= True
titleCount = 9
entryCount = 3

# option parsing
for option, arg in getopt.getopt(
    sys.argv[1:], "fdc:t:e:", ["flush", "count=", "titles=",
        "entries=", "no-entries"])[0]:
    if option in ('-f', '--flush'):
        date = datetime.now().strftime("%Y%m%d")
        time = datetime.now().strftime("%H%M%S")
        backup = "%s_%s.%s.pkl" % (filename, date, time)
        try:
            os.rename(pkl, backup)
            log.info("Renamed '%s' to '%s'." % (pkl, backup))
        except OSError:
            log.info("Nothing to flush.")
    if option in ('-c', '--count'):
        defaultCount = arg
    if option == '--no-entries':
        doEntries = False
    if option in ('-t', '--titles'):
        titleCount = int(arg)
    if option in ('-e', '--entries'):
        entryCount = int(arg)

x = XcretaRun()
if not os.path.exists(pkl):
    log.info('Havesting data from on-line sources...')
    #client = clients.TechnoratiFeeds()
    client = clients.BloglinesClient()
    feeds = client.getFeedList()
    log.debug("Feedster data length: %s" % len(feeds))
    x.harvest = harvesters.SynchHarvester(feeds)
    # save the data for potential future use
    pickle.dump(x.getPicklableHarvest(), open(pkl, 'w+'))
else:
    log.info('Getting havested info from old data...')
    x.harvest = pickle.load(open(pkl))

# Now print out some results
x.printTitles(titleCount)
if doEntries:
    x.printEntries(entryCount)
