import os
import sys
import getopt
import cPickle as pickle
from random import choice
from datetime import datetime

from twisted.internet import defer
from twisted.internet import reactor

from webxcreta import tagger
from webxcreta import clients
from webxcreta import schrobox
from webxcreta import harvesters
from webxcreta.utils import log

filename = 'var/harvested_data'
pkl = filename + '.pkl'
for option, arg in getopt.getopt(sys.argv[1:], "f", ["flush"])[0]:
    if option in ('-f', '--flush'):
        date = datetime.now().strftime("%Y%m%d")
        time = datetime.now().strftime("%H%M%S")
        backup = "%s_%s.%s.pkl" % (filename, date, time)
        try:
            os.rename(pkl, backup)
            log.info("Renamed '%s' to '%s'." % (pkl, backup))
        except OSError:
            log.info("Nothing to flush.")

def chewDataCallback(nil, harvest):
    tagger.tagData(harvest)
    return harvest

def printResultsCallback(harvest):
    '''
    Print a few of the first results.
    '''
    possible_titles = schrobox.TitleGenerator(harvest)
    possible_entries = schrobox.TextGenerator(harvest)

    for i in range(0,9):
        print "Possible title: %s" % possible_titles.next()
    print "\n\n"
    for i in range(0,3):
        print "Possible entry:\n\t%s\n\n" % possible_entries.next()
    return harvest

def preserveData(harvest):
    fake_harvest = harvesters.FakeHarvest()
    fake_harvest.data = harvest.data
    fake_harvest.titles = harvest.titles
    fake_harvest.entries = harvest.entries
    pickle.dump(fake_harvest, open(pkl, 'w+'))
    reactor.stop()

def run():
    if not os.path.exists(pkl):
        log.info('Asynchronously havesting data from on-line sources...')
        feedster = clients.FeedsterClient(count=500)
        feeds = feedster.getFeedList()
        max_hosts_in_queue = 50
        async_harvest = harvesters.AsynchHarvester()

        sem = defer.DeferredSemaphore(max_hosts_in_queue)
        deferreds = [sem.run(async_harvest.setupFeedDeferred, feed) 
            for feed in feeds]
        dl = defer.DeferredList(deferreds)
        dl.addCallback(async_harvest.handleAllGroups)
        dl.addErrback(async_harvest.handleFailure)
        dl.addCallback(chewDataCallback, async_harvest)
        dl.addCallback(printResultsCallback)
        # save the data for potential future use
        dl.addCallback(preserveData)
    else:
        log.info('Getting havested info from old data...')
        harvest = pickle.load(open(pkl))
        possible_titles = schrobox.TitleGenerator(harvest)
        possible_entries = schrobox.TextGenerator(harvest)

if __name__ == '__main__':
    run()
    reactor.run()
