import tagger
from data import Message
from harvesters import FakeHarvester
from schrobox import TitleGenerator
from schrobox import TextGenerator

class XcretaRun(object):
    '''
    Set up an email message:
    >>> from utils import FakeEmailMessage, extractEmailData
    >>> fe = FakeEmailMessage()
    >>> fe.Subject = 'Re: About last night'
    >>> fe.body = """Dear Mary,
    ... Yeah, so weren't those flying saucers crazy?!"""

    Extract data from it
    >>> data = extractEmailData(fe)

    Add the data to a harvester:
    >>> harvest = FakeHarvester()
    >>> harvest.data.append(data)

    Run webXcreta with the harvester:
    >>> x = XcretaRun()
    >>> x.setHarvest(harvest)
    >>> x.harvest.data[0]['title_tagged']
    [('Re', 'nn-tl'), (':', ':'), ('About', 'rb'), ('last', 'ap'), ('night', 'nn')]
    '''
    harvest = None

    def __init__(self):
        self._titles = iter([])
        self._entries = iter([])
        self._harvest = FakeHarvester()

    def setTitles(self, titles=None):
        if titles:
            self._titles = titles
        else:
            self._titles = TitleGenerator(self.harvest)

    def setEntries(self, entries=None):
        if entries:
            self._entries = entries
        else:
            self._entries = TextGenerator(self.harvest)

    def setHarvest(self, harvest):
        if not (harvest.entries or harvest.titles):
            tagger.tagData(harvest)
        self._harvest = harvest
        self.setTitles()
        self.setEntries()

    def getPicklableHarvest(self):
        fakeHarvest = FakeHarvester()
        fakeHarvest.data = self.harvest.data
        fakeHarvest.titles = self.harvest.titles
        fakeHarvest.entries = self.harvest.entries
        return fakeHarvest

    def printTitles(self, count):
        for i in xrange(count):
            print "Possible title: %s" % self.titles.next()
        print "\n"

    def printEntries(self, count):
        if not self.entries:
            log.error('No entries were generated for this post!')
        for i in xrange(count):
            print "Possible entry:\n\t%s\n\n" % self.entries.next()

    entries = property(lambda s: s._entries, setEntries)
    titles = property(lambda s: s._titles, setTitles)
    harvest = property(lambda s: s._harvest, setHarvest)

def _test():
    import doctest, base
    return doctest.testmod(base)

if __name__ == '__main__':
    _test()
