import re
import sys
from urllib2 import urlopen
from StringIO import StringIO
from itertools import chain

from elementtree.ElementTree import tostring, fromstring
from elementtree import HTMLTreeBuilder

from BeautifulSoup import BeautifulSoup, Comment

from zope.testbrowser.browser import Browser

from webxcreta.utils import log, recoverableError


XHTML = "{http://www.w3.org/1999/xhtml}"

SKIP_HOSTS = [
    'beppegrillo.it',
    'gigazine.net',
    'www.microsiervos.com',
    "japanese.engadget.com",
    "mysinablog.com",
    "blog.sina.com.cn",
]


def getPageHTML(url):
    return urlopen(url).read()


def cleanNamespaces(tree):
    for elem in tree.getiterator():
        if elem.tag.startswith(XHTML):
            elem.tag = elem.tag[len(XHTML):]
    return tree

SKIP_FEEDS = [
    "http://feeds.feedburner.com/martinvarsavsky/spanish",
    ]


class BaseClient(object):

    def __init__(self, url):
        self.url = url
        self.html = ''
        self.feeds = []
        self.makeTree()

    def makeTree(self):
        self.html = StringIO(getPageHTML(self.url))
        self.tree = cleanNamespaces(TidyHTMLTreeBuilder.parse(self.html))
        soup = BeautifulSoup(urlopen(url).read())
        # remove script tags and comments for easier parsing
        scripts = soup.findAll("script")
        comments = soup.findAll(text=lambda text:isinstance(text, Comment))
        for element in chain(scripts, comments):
            #import pdb;pdb.set_trace()
            element.extract()
        cleaned = soup.prettify()
        self.html = StringIO(cleaned)
        self.tree = HTMLTreeBuilder.parse(self.html)
        for elem in self.tree.getiterator():
            if elem.tag.startswith(XHTML):
                elem.tag = elem.tag[len(XHTML):]


class BloglinesClient(BaseClient):

    def __init__(self):
        url = 'http://www.blogstreet.com/top100.html'
        super(BloglinesClient, self).__init__(url)

    def makeTree(self):
        super(BloglinesClient, self).makeTree()
        tables = []
        for elem in self.tree.getiterator():
            if elem.tag == 'table':
                tables.append(elem)
        self.dataTable = tables[4]

    def getFeedList(self):
        feedCount = -1
        hrefCounter = 0
        for elem in self.dataTable.getiterator():
            if elem.tag == 'tr':
                hrefCounter = 0
                feedCount += 1
            if elem.tag == 'a':
                hrefCounter += 1
                if hrefCounter == 4:
                    self.feeds.append({
                        'link':elem.attrib['href'],
                        'order':feedCount,
                        'rank':elem.attrib['href']})
        return self.feeds


class FeedsterClient(BaseClient):

    def __init__(self, count=500):
        url = 'http://top500.feedster.com/?offset=0&limit=%s' % count
        super(FeedsterClient, self).__init__(url)

    def getFeedList(self):
        feedCount = 0
        hrefCount = 0
        self.feeds.insert(feedCount, {})
        for elem in self.tree.getiterator():
            if (elem.attrib.has_key('id') and
                elem.tag == 'div'):
                item = elem.attrib.get('id')
                if item == 'fivehundred-item':
                    feedCount += 1
                    self.feeds.insert(feedCount, {})
                    hrefCount = 0
                    #print " "
            if elem.tag == 'a' and hrefCount < 20:
                hrefCount += 1
                #print elem.attrib.get('href')
                #print hrefCount
                if hrefCount == 1:
                    link = elem.attrib['href'].split('?url=')
                    if len(link) > 1:
                        self.feeds[feedCount].setdefault(
                            'link', link[1].split('&content=summary')[0])
                        self.feeds[feedCount].setdefault(
                            'order', feedCount)
            # we know that the links number is in this div tag,
            # but the exact location is not consistent, so we
            # need to convert the whole element to a string and
            # use brute force to get it
            if (elem.tag == 'div' and
                elem.attrib.has_key('class') and
                elem.attrib.has_key('style') and
                elem.attrib.get('style') == 'width:450px;'):
                #divParts = re.split('[()]', tostring(elem))
                links = ''
                divParts = tostring(elem).split('adjusted links')
                if divParts:
                    partParts = divParts[0].split('(')
                    if partParts:
                        links = partParts[-1].strip().replace(',','')
                if links:
                    #pprint(links)
                    self.feeds[feedCount].setdefault('rank', links)
        # the first one's never an actual feed, so skip it
        return self.feeds[1:]


class TechnoratiClient(BaseClient):

    def __init__(self, page):
        self.page = page
        url = 'http://technorati.com/pop/blogs/?page=%i' % self.page
        super(TechnoratiClient, self).__init__(url)

    def makeTree(self):
        self.html = ''
        for page in xrange(10):
            pageNum = page + 1
            log.info('Getting page %i' % pageNum)
            self._scrape(self.url % pageNum)
        try:
            # wrap all the <ol> chunks in a div
            self.html = '<div>%s</div>' % self.html
            self.tree = cleanNamespaces(XML(self.html))
        except Exception, exp:
            print exp
            import pdb;pdb.set_trace()

    def _scrape(self, url):
        # need to fix up the brokenness in either technorati's HTML or in
        # ElementTidy, so we'll extract just the part of the HTML that we need
        # and make that our tree.
        html = getPageHTML(url)
        start = 0
        stop = 0
        htmlLines = html.split('\n')
        for index, line in enumerate(htmlLines):
            if '<ol class="whatsup latest">' in line:
                start = index
            if '</ol' in line:
                stop = index + 1
                break
        html = ''.join(htmlLines[start:stop])
        # bs cleanup
        html = html.replace('&raquo;', '')
        html = html.replace('&amp', '&amp;')
        # get rid of any trailing tags
        html = re.match('(.*</ol>)(\s*<div id="pages".*)',
                             html).groups()[0]
        html = html.replace("&squot;", "")
        html = html.replace("&#183;", "")
        self.html = html.replace("&amp;;", "&amp;")
        try:
            # do a test to make sure the HTML is parsable
            tree = cleanNamespaces(XML(html))
            self.html += html
        except Exception, exp:
            print exp
            import pdb;pdb.set_trace()

    def getFeedList(self):
        if self.feeds:
            return self.feeds
        blogURLs = [a.get('href')[len('/blogs/'):]
                    for a in self.tree.findall('.//h3/a')
                    if a.get('href').startswith('/blogs/')]
        for index, url in enumerate(blogURLs):
            if True in [url.startswith(x) for x in SKIP_HOSTS]:
                log.info("Skipping non-English blog ...")
                continue
            order = rank = (index + 1) + ((self.page - 1) * 10)
            #log.debug("found URL: %s" % url)
            sys.stdout.write(".")
            sys.stdout.flush()
            if not url.startswith("http://"):
                url = "http://%s" % url
            try:
                c = BaseClient(url)
            except Exception, e:
                sys.stdout.write("\n")
                msg = "Couldn't parse the HTML from %s." %  url
                recoverableError(msg, e)
                continue
            feedURLs = []
            for link in c.tree.findall('//link'):
                type = link.get('type')
                if type and 'application/' in type:
                    href = link.get('href')
                    if not href.startswith('http://'):
                        href = url + href
                    feedURLs.append(href)
            if feedURLs:
                self.feeds.append({
                    'link': feedURLs[0],
                    'order': order,
                    'rank': rank})
        sys.stdout.write("\n")
        return self.feeds


class TechnoratiFeeds(object):
    """
    A helper class that connects to multiple Technorati pages to get a list of
    top blog sites.
    """
    def getFeedList(self):
        feeds = []
        for i in xrange(10):
            pageNum = i + 1
            client = TechnoratiClient(pageNum)
            feeds.extend(client.getFeedList())
        return feeds


class BloglinesClient(object):
    """
    A Bloglines client.
    """
    urlTemplate = "http://beta.bloglines.com/topfeeds?page=%i"

    def getFeedBatch(self, page):
        feeds = []
        browser = Browser(self.urlTemplate % page)
        index = 0
        for link in browser.mech_browser.links():
            if dict(link.attrs).get("title") == "Subscribe to Feed":
                index += 1
                order = rank = (index + 1) + ((page - 1) * 100)
                feeds.append({
                    "link": link.url,
                    "order": order,
                    "rank": rank,
                    })
                sys.stdout.write(".")
                sys.stdout.flush()
        return feeds

    def getFeedList(self):
        feeds = []
        for i in xrange(10):
            pageNum = i + 1
            feeds.extend(self.getFeedBatch(pageNum))
        sys.stdout.write("\n")
        log.info("Got %i RSS feeds." % len(feeds))
        return feeds
