class Message(dict):
    '''
    This class is used to standardize the data that webXcreta will
    deal with, no matter the original source.

    >>> m = Message()
    >>> i = m.items()
    >>> i.sort()
    >>> i
    [('content', ''), ('link', 1), ('order', 1), ('rank', 1), ('title', '')]
    >>> m = Message(rank=200, title='Amazing Blog Entry',
    ...   content='Really crappy content')
    >>> i = m.items()
    >>> i.sort()
    >>> i
    [('content', 'Really crappy content'), ('link', 1), ('order', 1), ('rank', 200), ('title', 'Amazing Blog Entry')]

    >>> m['title']
    'Amazing Blog Entry'
    >>> m['content']
    'Really crappy content'

    >>> m.title
    'Amazing Blog Entry'
    >>> m.content
    'Really crappy content'

    ''' 
    def __init__(self, rank=1, title='', content='', link=1, order=1):
        data = {
            'rank'      : rank,
            'title'     : title,
            'content'   : content,
            'link'      : link,
            'order'     : order}
        super(Message, self).__init__(**data)
        for k, v in data.items():
            setattr(self, k, v)

def _test():
    import doctest, data
    return doctest.testmod()

if __name__ == '__main__':
    _test()
