from random import choice

from webxcreta import stats
from webxcreta.utils import log

# XXX this is a cheap hack until the code for the word filtering
# is cleaned up
def getWord(partOfSpeech, wordBag):
    try:
        return choice(wordBag[partOfSpeech])
    except:
        return getWord(partOfSpeech)

class TestBase(object):

    _data = ['apples', 'oranges', 'bananas', 'kiwis',
        'strawberries', 'mangos']

    choices = iter([])

    def makeChoice(self):
        while True:
            yield choice(self._data)

    def next(self):
        if not self.choices:
            print "self.choices is null; setting..."
            self.choices = self.makeChoice()
        return self.choices.next()

class BaseGenerator(object):

    def __init__(self):
        self.content = iter([])

    def next(self):
        if not self.content:
            log.debug("self.choices is null; setting...")
            self.content = self.makeChoice()
        return self.content.next()

    def __iter__(self):
        pass

    makeChoice = __iter__

class TitleGenerator(BaseGenerator):

    def __init__(self, harvest):
        super(TitleGenerator, self).__init__()
        # check to see if titles have already been weighed
        words = harvest.titles.get('wordBag')
        syntax = harvest.titles.get('syntaxBag')
        if not (words or syntax):
            log.info("'titles' attribute null; calculating weights...")
            words, syntax = stats.weighTitle(harvest)
        else:
            log.info("'titles' attribute found; skipping weight calculations...")
        self.wordBag = words
        self.syntaxBag = syntax
        
    def __iter__(self):
        while True:
            structure = choice(self.syntaxBag)
            newTitle = ' '.join([ choice(self.wordBag[partOfSpeech])
                for partOfSpeech in structure ]).title()
            if newTitle.strip() == ':':
                self.makeChoice()
            yield newTitle

    def next(self):
        return self.__iter__().next()

class TextGenerator(BaseGenerator):

    def __init__(self, harvest):
        super(TextGenerator, self).__init__()
        # check to see if entries have already been weighed
        words = harvest.entries.get('wordBag')
        syntax = harvest.entries.get('syntaxBag')
        lengths = harvest.entries.get('entryLengths')
        if not (words or syntax or lengths):
            log.info("'entries' attribute null; calculating weights...")
            words, syntax, lengths = stats.weighText(harvest)
        else:
            log.info("'entries' attribute found; skipping weight calculations...")
        self.wordBag = words
        self.syntaxBag = syntax
        self.textLengths = lengths

    def __iter__(self):
        while True:
            thisEntry = []
            # how long will the post be?
            sentenceCount = choice(self.textLengths)
            for sentence in range(0, sentenceCount):
                # what word-order will this sentence have?
                structure = choice(self.syntaxBag)
                # for each part of speech in the structure, let's pick 
                # out a word and construct thte sentence.
                thisSentence = [ choice(self.wordBag[partOfSpeech])
                    for partOfSpeech in structure ]
                # now let's format the sentence
                thisSentence = ' '.join(thisSentence).capitalize()
                thisSentence = thisSentence.replace(' ,', ',')
                thisSentence = thisSentence.replace(' .', '.')
                thisSentence = thisSentence.replace('&', 'and')
                thisEntry.append(thisSentence)
            yield ' '.join(thisEntry)

    def next(self):
        return self.__iter__().next()


