import math

from webxcreta.utils import log

def getMultiplier(feedCount, rank, highestRank):
    '''
    This funtion is completely arbitrary. Using a simple linear
    relationship between the number of links and the amount of
    content and syntax contributed resulted in an over-whelming
    bias. Conversely, taking the log of that number overly represented
    the lower end of the spectrum. However, there was a "preceived"
    happy medium when taking the square of that.

    This has me very curious about potential socio-political
    applications... where no voice, however small, is completely
    drowned out by the majority... where no majority can obtain an
    absolute foot-hold, a dictorial popular rule...
    '''
    rankMultiplier =  feedCount * 2
    ratio = rank/float(highestRank)
    raw = ratio * 100 * rankMultiplier
    smoothed = int(math.log(raw) ** 2)
    tmpl = ("feedCount: %s; rank: %s; highestRank: %s; rankMultiplier: %s; "
        + "ratio: %s; raw: %s; smoothed: %s")
    msg = tmpl % (feedCount, rank, highestRank, rankMultiplier, ratio,
              raw, smoothed)
    log.debug(msg)
    return smoothed

def weighTitle(harvest):
    # word bag will be a dictionary with parts of speech as the indices
    # and lists of words as the values
    # syntax bag will be a list of sentence/fragment word orders
    wordBag = {}
    syntaxBag = []
    for entry in harvest.data:
        log.debug("Full entry:\n%s" % entry)
        title = entry['title_tagged']
        rank = entry['rank']
        log.debug(title)
        if title:
            log.debug("Highest rank: %s" % harvest.highestRank)
            multiplier = getMultiplier(len(harvest.data), rank,
                harvest.highestRank)
            log.debug("Rank: %s" % rank)
            log.debug("Multiplier: %s" % multiplier)
            for word, partOfSpeech in title*multiplier:
                nil = wordBag.setdefault(partOfSpeech, [])
                wordBag[partOfSpeech].append(word)
            for count in range(0, multiplier):
                syntaxBag.append([ partOfSpeech
                    for word, partOfSpeech in title ])
    log.debug("Title word bag: %s" % wordBag)
    log.debug("Title syntax bag: %s" % syntaxBag)

    harvest.titles['wordBag'] = wordBag
    harvest.titles['syntaxBag'] = syntaxBag
    return (wordBag, syntaxBag)

def weighText(harvest):
    wordBag = {}
    syntaxBag = []
    entryLengths = []
    for entry in harvest.data:
        content = entry['contentTagged']
        rank = entry['rank']
        if content:
            multiplier = getMultiplier(len(harvest.data), rank,
                harvest.highestRank)
            entryLengths.extend([len(content)]*multiplier)
            # now, for each sentence
            for sentence in content:
                log.debug("Sentence:\n %s" % sentence)
                for word, partOfSpeech in sentence*multiplier:
                    nil = wordBag.setdefault(partOfSpeech, [])
                    wordBag[partOfSpeech].append(word)
                for count in range(0, multiplier):
                    parts = []
                    for word, partOfSpeech in sentence:
                        parts.append(partOfSpeech)
                        syntaxBag.append(parts)

    harvest.entries['wordBag'] = wordBag
    harvest.entries['syntaxBag'] = syntaxBag
    harvest.entries['entryLengths'] = entryLengths
    return (wordBag, syntaxBag, entryLengths)

