import re
import sys
import logging as log
import sgmllib, string

from data import Message


class StrippingParser(sgmllib.SGMLParser):
    
    from htmlentitydefs import entitydefs # replace entitydefs from sgmllib
    
    def __init__(self):
        sgmllib.SGMLParser.__init__(self)
        self.result = ""
        self.endTagList = []
        
    def handle_data(self, data):
        if data:
            self.result = self.result + data

    def handle_charref(self, name):
        self.result = "%s&#%s;" % (self.result, name)
        
    def handle_entityref(self, name):
        if self.entitydefs.has_key(name):
            x = ';'
        else:
            # this breaks nonstandard entities that end with ';'
            x = ''
        self.result = "%s&%s%s" % (self.result, name, x)

    valid_tags = ('b', 'a', 'i', 'br', 'p')
    
    def unknown_starttag(self, tag, attrs):
        """ Delete all tags except for legal ones """
        if tag in self.valid_tags:
            self.result = self.result + '<' + tag
            for k, v in attrs:
                if string.lower(k[0:2]) != 'on' and string.lower(v[0:10]) != 'javascript':
                    self.result = '%s %s="%s"' % (self.result, k, v)
            endTag = '</%s>' % tag
            self.endTagList.insert(0,endTag)
            self.result = self.result + '>'
                
    def unknown_endtag(self, tag):
        if tag in self.valid_tags:
            self.result = "%s </%s>" % (self.result, tag)
            remTag = '</%s>' % tag
            self.endTagList.remove(remTag)

    def cleanup(self):
        """ Append missing closing tags """
        for j in range(len(self.endTagList)):
                self.result = self.result + self.endTagList[j]

log.basicConfig(level=log.INFO,
    format='%(levelname)-8s %(message)s',
    stream=sys.stdout,
)

def stripper(s):
    parser = StrippingParser()
    parser.valid_tags = []
    parser.feed(s)
    parser.close()
    parser.cleanup()
    return parser.result

def stripCrap(t):
    t = stripper(t).replace('\n', ' ')
    return re.sub('[^A-Za-z0-9,.;:"\'?! ]', '', t)

class FakeEmailMessage(object):
    '''
    >>> fe = FakeEmailMessage()
    >>> fe.get('Subject')
    >>> fe.getPayload()
    ''
    >>> fe.Subject = 'Re: About last night'
    >>> fe.get('Subject')
    'Re: About last night'
    >>> fe.body = "Yeah, so weren't those flying saucers crazy?!"
    >>> fe.getPayload()
    "Yeah, so weren't those flying saucers crazy?!"
    '''

    def __init__(self, *args, **kws):
        super(FakeEmailMessage, self).__init__(*args, **kws)
        self.body = ''

    def getPayload(self):
        return self.body

    def get(self, attr):
        return getattr(self, attr, None)

def skipLines(text, skip=0):
    '''
    >>> text = 'Testing'
    >>> skipLines(text)
    'Testing'
    >>> skipLines(text, skip=1)
    ''

    >>> text = """Testing
    ... 1, 2, 3
    ... """
    >>> print skipLines(text)
    Testing
    1, 2, 3
    >>> print skipLines(text, skip=1)
    1, 2, 3

    '''
    textLines = text.split('\n')[skip:]
    return '\n'.join(textLines).strip()

def extractEmailData(msg, skip_lines=1):
    '''
    >>> fe = FakeEmailMessage()
    >>> fe.Subject = 'Re: About last night'
    >>> fe.body = """Dear Mary,
    ... Yeah, so weren't those flying saucers crazy?!"""
    
    >>> data = extractEmailData(fe)
    >>> data.title
    'Re: About last night'
    >>> data.content
    "Yeah, so weren't those flying saucers crazy?!"
    '''
    title = msg.get('Subject') or ' '
    pl = msg.getPayload()

    # not supporting multi-part messages
    if isinstance(pl, list):
        return

    content = skipLines(pl, skip=skip_lines)
    content = stripCrap(content)
    if content:
        return Message(title=title, content=content)

def recoverableError(msg, exception):
    if msg:
        log.info(msg)
    if exception:
        log.error(exception)
    log.info("Skipping ...")

def _test():
    import doctest, utils
    return doctest.testmod()

if __name__ == '__main__':
    _test()


